#Date: 20 June 2010
#Written by Bin Han and Tomasz Neugebauer
#Concordia University Libraries
#thesis_advisors_name, thesis_degree_name and department, are custom fields that are added to eprint_fields.pl so that EPrint follows ETD-MS model
#on line 173, you need to supply a Library ID for one of the identifier fields (required by Library and Archives Canada harvesting)

package EPrints::Plugin::Export::ETD_MS;

use EPrints::Plugin::Export;
@ISA = ( "EPrints::Plugin::Export" );

use strict;

sub new
{
        my( $class, %opts ) = @_;

		my $self = $class->SUPER::new( %opts );

        $self->{name} = "ETD_MS";
		$self->{accept} = ['dataobj/eprint', 'list/eprint'];
		$self->{visible} = "all";
        $self->{suffix} = ".xml";
        $self->{mimetype} = "text/xml; charset=utf-8";

		$self->{xmlns} = "http://www.ndltd.org/standards/metadata/etdms/1.0/";
		$self->{schemaLocation} = "http://www.ndltd.org/standards/metadata/etdms/1.0/etdms.xsd";

        return $self;
}

sub output_list
{
	my( $plugin, %opt ) = @_;

	my $dc = $plugin->{session}->make_element(
		"collection",
	);

	foreach my $dataobj ($opt{list}->get_records)
	{
		$dc->appendChild( $plugin->xml_dataobj($dataobj) );
	}

	#filehandle provided by the command line export tool
	if (defined $opt{fh})
	{
		print {$opt{fh}} EPrints::XML::to_string($dc);
		return undef;
	}
	else
	{
		return EPrints::XML::to_string($dc);
	}
}

sub output_dataobj
{
	my( $plugin, $dataobj ) = @_;

	my $xml = $plugin->xml_dataobj( $dataobj );

	return EPrints::XML::to_string( $xml );
}

sub xml_dataobj
{
	my( $plugin, $dataobj ) = @_;

	my $data = $plugin->convert_dataobj( $dataobj );

	my $dc = $plugin->{session}->make_element(
			 "etd_ms:thesis",
			 "xmlns:etd_ms" => "http://www.ndltd.org/standards/metadata/etdms/1.0/",
			 "xmlns:xsi" => "http://www.w3.org/2001/XMLSchema-instance",
			 "xsi:schemaLocation" => "http://www.ndltd.org/standards/metadata/etdms/1.0/ http://www.ndltd.org/standards/metadata/etdms/1.0/etdms.xsd"
			 );

	# turn the list of pairs into XML blocks (indented by 8) and add them to the DC element.
	foreach( @{$data} )
	{
		$dc->appendChild( $_ );
		# produces <key>value</key>
	}

	return $dc;
}

# Convert eprint object into a array.
sub convert_dataobj
{
	my( $plugin, $eprint ) = @_;

	my @dcdata = ();
	if ( $eprint->exists_and_set( "type" ) && $eprint->get_value( "type" ) eq "thesis" )
	{

		# <title />
		if( $eprint->exists_and_set( "title" ) )
		{
			my $title = $eprint->get_value( "title" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:title", $title) );
		}

		# <creators />
		if( $eprint->exists_and_set( "creators_name" ) )
		{
			my $creators = $eprint->get_value( "creators_name" );
			foreach my $creator ( @{$creators} )
			{
				push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:creator", EPrints::Utils::make_name_string($creator)) );
			}
		}

		# <subject />
		my $subject = "";
		if( $eprint->exists_and_set( "keywords" ) )
		{
			$subject = $eprint->get_value( "keywords" );
		}
		push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:subject", $subject) );

		# <description />
		if( $eprint->exists_and_set( "abstract" ) )
		{
			my $description = $eprint->get_value( "abstract" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:description", $description) );
		}

		# <contributor role="advisor" />
		if( $eprint->exists_and_set( "thesis_advisors_name" ) )
		{
			my $contributors = $eprint->get_value( "thesis_advisors_name" );
			foreach my $contributor ( @{$contributors} )
			{
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:contributor",
					EPrints::Utils::make_name_string($contributor),
					"role"=>"advisor")
					);
			}
			#push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:contributor", EPrints::Utils::tree_to_utf8($eprint->render_value("thesis_advisor_name")), "role"=>"advisor") );
		}


		# <date />
		if( $eprint->exists_and_set( "date" ) )
		{
			my $date = $eprint->get_value( "date" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:date", $date) );
		}

		# <type />
		if( $eprint->exists_and_set( "type" ) )
		{
			my $type = $eprint->get_value( "type" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:type", "Electronic Thesis or Dissertation") );
		}

		# <format />
		my @documents = $eprint->get_all_documents();
		my $mimetypes = $plugin->{session}->get_repository->get_conf( "oai", "mime_types" );
		foreach my $document ( @documents )
		{
			my $format = $mimetypes->{$document->get_value("format")};
			$format = $document->get_value("format") unless defined $format;
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:format", $format ) );
		}

		# <identifier />
		if( $eprint->exists_and_set( "eprintid" ) )
		{
			my $identifier = "[insert-library-specific-identifier]".$eprint->get_value( "eprintid" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:identifier", $identifier) );
		}

		# <identifier /> <language />
		foreach my $document ( @documents )
		{
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:identifier", $document->get_url() ) );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:language", $document->get_value("language") ) );
		}

		# <publisher />
		if( $eprint->exists_and_set( "publisher" ) )
		{
			my $publisher = $eprint->get_value( "publisher" );
			push @dcdata, ( $plugin->{session}->render_data_element(8, "etd_ms:publisher", $publisher) );
		}

		# <degree />
		my $dc_degree = $plugin->{session}->make_element( "etd_ms:degree" );

		# <name />
		if( $eprint->exists_and_set( "thesis_degree_name" ) )
		{
			my $degree_name = $eprint->get_value( "thesis_degree_name" );
			$dc_degree->appendChild( $plugin->{session}->render_data_element(8, "etd_ms:name", $degree_name) );
		}

		# <level />
		if( $eprint->exists_and_set( "thesis_type" ) )
		{
			my $degree_level = $eprint->get_value( "thesis_type" );
			$dc_degree->appendChild( $plugin->{session}->render_data_element(8, "etd_ms:level", $degree_level) );
		}

		# <department />
		if( $eprint->exists_and_set( "department" ) )
		{
			my $degree_dep = $eprint->get_value( "department" );
			$dc_degree->appendChild( $plugin->{session}->render_data_element(8, "etd_ms:discipline", $degree_dep) );
		}

		# <grantor />
		if( $eprint->exists_and_set( "institution" ) )
		{
			my $degree_grantor = $eprint->get_value( "institution" );
			$dc_degree->appendChild( $plugin->{session}->render_data_element(8, "etd_ms:grantor", $degree_grantor) );
		}

		push @dcdata, ( $dc_degree );
	}
	return \@dcdata;
}

1;